@@ -25,31 +25,44 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <string.h>
+#include <errno.h>
+#include <stdlib.h>       
+#include <fcntl.h>
 
 #include "gtkprintoperation-private.h"
 #include "gtkmarshal.h"
 #include "gtkmessagedialog.h"
 
+#include <cairo-pdf.h>
+#include <cairo-ps.h>
 #include "gtkprintunixdialog.h"
 #include "gtkpagesetupunixdialog.h"
 #include "gtkprintbackend.h"
 #include "gtkprinter.h"
+#include "gtkprinter-private.h"
 #include "gtkprintjob.h"
+#include "gtklabel.h"
 #include "gtkalias.h"
 #include "gtkintl.h"
 
-typedef struct {
-  GtkPrintJob *job;         /* the job we are sending to the printer */
-  gulong job_status_changed_tag;
+typedef struct 
+{
   GtkWindow *parent;        /* just in case we need to throw error dialogs */
   GMainLoop *loop;
   gboolean data_sent;
+
+  /* Real printing (not preview) */
+  GtkPrintJob *job;         /* the job we are sending to the printer */
+  cairo_surface_t *surface;
+  gulong job_status_changed_tag;
+
+  
 } GtkPrintOperationUnix;
 
 typedef struct _PrinterFinder PrinterFinder;
 
 static void printer_finder_free (PrinterFinder *finder);
-static void find_printer        (const char    *printer,
+static void find_printer        (const gchar   *printer,
 				 GFunc          func,
 				 gpointer       data);
 
@@ -58,21 +71,24 @@ unix_start_page (GtkPrintOperation *op,
 		 GtkPrintContext   *print_context,
 		 GtkPageSetup      *page_setup)
 {
+  GtkPrintOperationUnix *op_unix;  
   GtkPaperSize *paper_size;
   cairo_surface_type_t type;
-  double w, h;
+  gdouble w, h;
 
+  op_unix = op->priv->platform_data;
+  
   paper_size = gtk_page_setup_get_paper_size (page_setup);
 
   w = gtk_paper_size_get_width (paper_size, GTK_UNIT_POINTS);
   h = gtk_paper_size_get_height (paper_size, GTK_UNIT_POINTS);
   
-  type = cairo_surface_get_type (op->priv->surface);
+  type = cairo_surface_get_type (op_unix->surface);
 
   if (type == CAIRO_SURFACE_TYPE_PS)
-    cairo_ps_surface_set_size (op->priv->surface, w, h);
+    cairo_ps_surface_set_size (op_unix->surface, w, h);
   else if (type == CAIRO_SURFACE_TYPE_PDF)
-    cairo_pdf_surface_set_size (op->priv->surface, w, h);
+    cairo_pdf_surface_set_size (op_unix->surface, w, h);
 }
 
 static void
@@ -81,7 +97,7 @@ unix_end_page (GtkPrintOperation *op,
 {
   cairo_t *cr;
 
-  cr = gtk_print_context_get_cairo (print_context);
+  cr = gtk_print_context_get_cairo_context (print_context);
   cairo_show_page (cr);
 }
 
@@ -98,9 +114,118 @@ op_unix_free (GtkPrintOperationUnix *op_unix)
   g_free (op_unix);
 }
 
+static gchar *
+shell_command_substitute_file (const gchar *cmd,
+			       const gchar *filename)
+{
+  const gchar *inptr, *start;
+  gchar *result;
+  GString *final;
+
+  g_return_val_if_fail (cmd != NULL, NULL);
+  g_return_val_if_fail (filename != NULL, NULL);
+
+  result = NULL;
+  final = g_string_new (NULL);
+
+  start = inptr = cmd;
+
+  while ((inptr = strchr (inptr, '%')) != NULL) 
+    {
+      g_string_append_len (final, start, inptr - start);
+      inptr++;
+      switch (*inptr) 
+        {
+          case 'f':
+            g_string_append (final, filename ? filename : "");
+            break;
+
+          case '%':
+            g_string_append_c (final, '%');
+            break;
+
+          default:
+            g_string_append_c (final, '%');
+            if (*inptr)
+              g_string_append_c (final, *inptr);
+            break;
+        }
+      if (*inptr)
+        inptr++;
+      start = inptr;
+    }
+  g_string_append (final, start);
+
+  result = final->str;
+
+  g_string_free (final, FALSE);
+
+  return result;
+}
+
+void
+_gtk_print_operation_platform_backend_launch_preview (GtkPrintOperation *op,
+						      cairo_surface_t   *surface,
+						      GtkWindow         *parent,
+						      const gchar       *filename)
+{
+  gint argc;
+  gchar **argv;
+  gchar *cmd;
+  gchar *preview_cmd;
+  GtkSettings *settings;
+  gchar *quoted_filename;
+  GdkScreen *screen;
+  GError *error = NULL;
+
+  cairo_surface_destroy (surface);
+ 
+  settings = gtk_settings_get_default ();
+  g_object_get (settings, "gtk-print-preview-command", &preview_cmd, NULL);
+
+  quoted_filename = g_shell_quote (filename);
+  cmd = shell_command_substitute_file (preview_cmd, quoted_filename);
+  g_shell_parse_argv (cmd, &argc, &argv, &error);
+
+  if (error != NULL)
+    goto out;
+
+  if (parent)
+    screen = gtk_window_get_screen (parent);
+  else
+    screen = gdk_screen_get_default ();
+  
+  gdk_spawn_on_screen (screen, NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error);
+
+ out:
+  if (error != NULL)
+    {
+      GtkWidget *edialog;
+      edialog = gtk_message_dialog_new (parent, 
+                                        GTK_DIALOG_DESTROY_WITH_PARENT,
+                                        GTK_MESSAGE_ERROR,
+                                        GTK_BUTTONS_CLOSE,
+                                        _("Error launching preview") /* FIXME better text */);
+      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (edialog),
+                                                "%s", error->message);
+      gtk_window_set_modal (GTK_WINDOW (edialog), TRUE);
+      g_signal_connect (edialog, "response",
+                        G_CALLBACK (gtk_widget_destroy), NULL);
+
+      gtk_window_present (GTK_WINDOW (edialog));
+
+      g_error_free (error); 
+   } 
+
+  g_free (cmd);
+  g_free (quoted_filename);
+  g_free (preview_cmd);
+  g_strfreev (argv);
+}
+
 static void
 unix_finish_send  (GtkPrintJob *job,
-                   void        *user_data, 
+                   gpointer     user_data, 
                    GError      *error)
 {
   GtkPrintOperationUnix *op_unix;
@@ -125,24 +250,32 @@ unix_finish_send  (GtkPrintJob *job,
     }
 
   op_unix->data_sent = TRUE;
+
   if (op_unix->loop)
     g_main_loop_quit (op_unix->loop);
 }
 
 static void
 unix_end_run (GtkPrintOperation *op,
-	      gboolean wait)
+	      gboolean           wait,
+	      gboolean           cancelled)
 {
   GtkPrintOperationUnix *op_unix = op->priv->platform_data;
 
+  cairo_surface_finish (op_unix->surface);
+  
+  if (cancelled)
+    return;
+
   if (wait)
     op_unix->loop = g_main_loop_new (NULL, FALSE);
   
   /* TODO: Check for error */
-  gtk_print_job_send (op_unix->job,
-                      unix_finish_send, 
-                      op_unix, NULL,
-		      NULL);
+  if (op_unix->job != NULL)
+    gtk_print_job_send (op_unix->job,
+                        unix_finish_send, 
+                        op_unix, NULL,
+		        NULL);
 
   if (wait)
     {
@@ -169,11 +302,19 @@ get_print_dialog (GtkPrintOperation *op,
                   GtkWindow         *parent)
 {
   GtkPrintOperationPrivate *priv = op->priv;
-  GtkWidget *pd;
+  GtkWidget *pd, *label;
   GtkPageSetup *page_setup;
+  const gchar *custom_tab_label;
 
   pd = gtk_print_unix_dialog_new (NULL, parent);
 
+  gtk_print_unix_dialog_set_manual_capabilities (GTK_PRINT_UNIX_DIALOG (pd),
+						 GTK_PRINT_CAPABILITY_PAGE_SET |
+						 GTK_PRINT_CAPABILITY_COPIES |
+						 GTK_PRINT_CAPABILITY_COLLATE |
+						 GTK_PRINT_CAPABILITY_REVERSE |
+						 GTK_PRINT_CAPABILITY_SCALE);
+
   if (priv->print_settings)
     gtk_print_unix_dialog_set_settings (GTK_PRINT_UNIX_DIALOG (pd),
 					priv->print_settings);
@@ -186,13 +327,34 @@ get_print_dialog (GtkPrintOperation *op,
                                         page_setup);
   g_object_unref (page_setup);
 
+  g_signal_emit_by_name (op, "create-custom-widget",
+			 &priv->custom_widget);
+
+  if (priv->custom_widget) 
+    {
+      custom_tab_label = priv->custom_tab_label;
+      
+      if (custom_tab_label == NULL)
+	{
+	  custom_tab_label = g_get_application_name ();
+	  if (custom_tab_label == NULL)
+	    custom_tab_label = _("Application");
+	}
+
+      label = gtk_label_new (custom_tab_label);
+      
+      gtk_print_unix_dialog_add_custom_tab (GTK_PRINT_UNIX_DIALOG (pd),
+					    priv->custom_widget, label);
+    }
+  
   return pd;
 }
   
-typedef struct {
+typedef struct 
+{
   GtkPrintOperation           *op;
   gboolean                     do_print;
-  GError                     **error;
+  gboolean                     do_preview;
   GtkPrintOperationResult      result;
   GtkPrintOperationPrintFunc   print_cb;
   GDestroyNotify               destroy;
@@ -211,79 +373,77 @@ print_response_data_free (gpointer data)
 
 static void
 finish_print (PrintResponseData *rdata,
-	      GtkPrinter *printer,
-	      GtkPageSetup *page_setup,
-	      GtkPrintSettings *settings)
+	      GtkPrinter        *printer,
+	      GtkPageSetup      *page_setup,
+	      GtkPrintSettings  *settings)
 {
   GtkPrintOperation *op = rdata->op;
   GtkPrintOperationPrivate *priv = op->priv;
-
-  priv->start_page = unix_start_page;
-  priv->end_page = unix_end_page;
-  priv->end_run = unix_end_run;
+  GtkPrintJob *job;
   
   if (rdata->do_print)
     {
-      GtkPrintOperationUnix *op_unix;
-
       gtk_print_operation_set_print_settings (op, settings);
-      
-      op_unix = g_new0 (GtkPrintOperationUnix, 1);
-      op_unix->job = gtk_print_job_new (priv->job_name,
-					printer,
-					settings,
-					page_setup);
+      priv->print_context = _gtk_print_context_new (op);
+      _gtk_print_context_set_page_setup (priv->print_context, page_setup);
 
-      gtk_print_job_set_track_print_status (op_unix->job, priv->track_print_status);
-      
-      rdata->op->priv->surface = gtk_print_job_get_surface (op_unix->job, rdata->error);
-      if (op->priv->surface == NULL)
+      if (!rdata->do_preview)
         {
-	  rdata->do_print = FALSE;
-	  op_unix_free (op_unix);
-	  rdata->result = GTK_PRINT_OPERATION_RESULT_ERROR;
-	  goto out;
-	}
+	  GtkPrintOperationUnix *op_unix;
+	  cairo_t *cr;
+	  
+	  op_unix = g_new0 (GtkPrintOperationUnix, 1);
+	  priv->platform_data = op_unix;
+	  priv->free_platform_data = (GDestroyNotify) op_unix_free;
+	  op_unix->parent = rdata->parent;
+	  
+	  priv->start_page = unix_start_page;
+	  priv->end_page = unix_end_page;
+	  priv->end_run = unix_end_run;
+	  
+	  job = gtk_print_job_new (priv->job_name, printer, settings, page_setup);
+          op_unix->job = job;
+          gtk_print_job_set_track_print_status (job, priv->track_print_status);
+	  
+	  op_unix->surface = gtk_print_job_get_surface (job, &priv->error);
+	  if (op_unix->surface == NULL) 
+            {
+	      rdata->result = GTK_PRINT_OPERATION_RESULT_ERROR;
+	      rdata->do_print = FALSE;
+	      goto out;
+            }
+	  
+	  cr = cairo_create (op_unix->surface);
+	  gtk_print_context_set_cairo_context (priv->print_context, cr, 72, 72);
+	  cairo_destroy (cr);
 
-      _gtk_print_operation_set_status (op, gtk_print_job_get_status (op_unix->job), NULL);
-      op_unix->job_status_changed_tag =
-	g_signal_connect (op_unix->job, "status_changed",
-			  G_CALLBACK (job_status_changed_cb), op);
-      
-      op_unix->parent = rdata->parent;
-
-      priv->dpi_x = 72;
-      priv->dpi_y = 72;
- 
-      priv->platform_data = op_unix;
-      priv->free_platform_data = (GDestroyNotify) op_unix_free;
-
-      priv->print_pages = op_unix->job->print_pages;
-      priv->page_ranges = op_unix->job->page_ranges;
-      priv->num_page_ranges = op_unix->job->num_page_ranges;
-  
-      priv->manual_num_copies = op_unix->job->num_copies;
-      priv->manual_collation = op_unix->job->collate;
-      priv->manual_reverse = op_unix->job->reverse;
-      priv->manual_page_set = op_unix->job->page_set;
-      priv->manual_scale = op_unix->job->scale;
-      priv->manual_orientation = op_unix->job->rotate_to_orientation;
+          _gtk_print_operation_set_status (op, gtk_print_job_get_status (job), NULL);
+	  
+          op_unix->job_status_changed_tag =
+	    g_signal_connect (job, "status-changed",
+			      G_CALLBACK (job_status_changed_cb), op);
+	  
+          priv->print_pages = job->print_pages;
+          priv->page_ranges = job->page_ranges;
+          priv->num_page_ranges = job->num_page_ranges;
+	  
+          priv->manual_num_copies = job->num_copies;
+          priv->manual_collation = job->collate;
+          priv->manual_reverse = job->reverse;
+          priv->manual_page_set = job->page_set;
+          priv->manual_scale = job->scale;
+          priv->manual_orientation = job->rotate_to_orientation;
+        }
     } 
-
- out:  
+ out:
   if (rdata->print_cb)
-    {
-      if (rdata->do_print)
-        rdata->print_cb (op, FALSE); 
-      else
-       _gtk_print_operation_set_status (op, GTK_PRINT_STATUS_FINISHED_ABORTED, NULL); 
-    }
+    rdata->print_cb (op, rdata->parent, rdata->do_print, rdata->result); 
 
   if (rdata->destroy)
     rdata->destroy (rdata);
 }
 
-static void
+static void 
 handle_print_response (GtkWidget *dialog,
 		       gint       response,
 		       gpointer   data)
@@ -296,30 +456,41 @@ handle_print_response (GtkWidget *dialog,
 
   if (response == GTK_RESPONSE_OK)
     {
-      rdata->result = GTK_PRINT_OPERATION_RESULT_APPLY;
-
       printer = gtk_print_unix_dialog_get_selected_printer (GTK_PRINT_UNIX_DIALOG (pd));
-      if (printer == NULL)
-	goto out;
-      
-      rdata->do_print = TRUE;
 
+      rdata->result = GTK_PRINT_OPERATION_RESULT_APPLY;
+      rdata->do_preview = FALSE;
+      if (printer != NULL)
+	rdata->do_print = TRUE;
+    } 
+  else if (response == GTK_RESPONSE_APPLY)
+    {
+      /* print preview */
+      rdata->result = GTK_PRINT_OPERATION_RESULT_APPLY;
+      rdata->do_preview = TRUE;
+      rdata->do_print = TRUE;
+    }
+
+  if (rdata->do_print)
+    {
       settings = gtk_print_unix_dialog_get_settings (GTK_PRINT_UNIX_DIALOG (pd));
       page_setup = gtk_print_unix_dialog_get_page_setup (GTK_PRINT_UNIX_DIALOG (pd));
-    } 
-
- out:  
+      
+      g_signal_emit_by_name (rdata->op, "custom-widget-apply", rdata->op->priv->custom_widget);
+    }
+  
   finish_print (rdata, printer, page_setup, settings);
 
   if (settings)
     g_object_unref (settings);
-  
+    
   gtk_widget_destroy (GTK_WIDGET (pd));
+ 
 }
 
 
 static void
-found_printer (GtkPrinter *printer,
+found_printer (GtkPrinter        *printer,
 	       PrintResponseData *rdata)
 {
   GtkPrintOperation *op = rdata->op;
@@ -330,7 +501,8 @@ found_printer (GtkPrinter *printer,
   if (rdata->loop)
     g_main_loop_quit (rdata->loop);
 
-  if (printer != NULL) {
+  if (printer != NULL) 
+    {
       rdata->result = GTK_PRINT_OPERATION_RESULT_APPLY;
 
       rdata->do_print = TRUE;
@@ -360,24 +532,24 @@ found_printer (GtkPrinter *printer,
 
 void
 _gtk_print_operation_platform_backend_run_dialog_async (GtkPrintOperation          *op,
+							gboolean                    show_dialog,
                                                         GtkWindow                  *parent,
 							GtkPrintOperationPrintFunc  print_cb)
 {
   GtkWidget *pd;
   PrintResponseData *rdata;
-  const char *printer_name;
+  const gchar *printer_name;
 
   rdata = g_new (PrintResponseData, 1);
   rdata->op = g_object_ref (op);
   rdata->do_print = FALSE;
   rdata->result = GTK_PRINT_OPERATION_RESULT_CANCEL;
-  rdata->error = NULL;
   rdata->print_cb = print_cb;
   rdata->parent = parent;
   rdata->loop = NULL;
   rdata->destroy = print_response_data_free;
   
-  if (op->priv->show_dialog)
+  if (show_dialog)
     {
       pd = get_print_dialog (op, parent);
       gtk_window_set_modal (GTK_WINDOW (pd), TRUE);
@@ -393,34 +565,94 @@ _gtk_print_operation_platform_backend_run_dialog_async (GtkPrintOperation
       if (op->priv->print_settings)
 	printer_name = gtk_print_settings_get_printer (op->priv->print_settings);
       
-      find_printer (printer_name,
-		    (GFunc) found_printer, rdata);
+      find_printer (printer_name, (GFunc) found_printer, rdata);
     }
 }
 
+cairo_surface_t *
+_gtk_print_operation_platform_backend_create_preview_surface (GtkPrintOperation *op,
+							      GtkPageSetup      *page_setup,
+							      gdouble           *dpi_x,
+							      gdouble           *dpi_y,
+							      gchar            **target)
+{
+  gchar *tmp_dir, *dir_template, *preview_filename;
+  GtkPaperSize *paper_size;
+  gdouble w, h;
+  
+  dir_template = g_build_filename (g_get_tmp_dir (), "print-preview-XXXXXX", NULL);
+
+  /* use temp dirs because apps like evince need to have extensions
+   * to determine the mime type 
+   */
+  tmp_dir = mkdtemp(dir_template);
+  preview_filename = g_build_filename (tmp_dir, 
+                                      "Print Preview.pdf",
+                                      NULL);
+  g_free (dir_template);
+  *target = preview_filename;
+  
+  paper_size = gtk_page_setup_get_paper_size (page_setup);
+  w = gtk_paper_size_get_width (paper_size, GTK_UNIT_POINTS);
+  h = gtk_paper_size_get_height (paper_size, GTK_UNIT_POINTS);
+    
+  *dpi_x = *dpi_y = 72;
+  return cairo_pdf_surface_create (preview_filename, w, h);
+}
+
+void
+_gtk_print_operation_platform_backend_preview_start_page (GtkPrintOperation *op,
+							  cairo_surface_t   *surface,
+							  cairo_t           *cr)
+{
+}
+
+void
+_gtk_print_operation_platform_backend_preview_end_page (GtkPrintOperation *op,
+							cairo_surface_t   *surface,
+							cairo_t           *cr)
+{
+  cairo_show_page (cr);
+}
+
+void
+_gtk_print_operation_platform_backend_resize_preview_surface (GtkPrintOperation *op,
+							      GtkPageSetup      *page_setup,
+							      cairo_surface_t   *surface)
+{
+  GtkPaperSize *paper_size;
+  gdouble w, h;
+  
+  paper_size = gtk_page_setup_get_paper_size (page_setup);
+  w = gtk_paper_size_get_width (paper_size, GTK_UNIT_POINTS);
+  h = gtk_paper_size_get_height (paper_size, GTK_UNIT_POINTS);
+  cairo_pdf_surface_set_size (surface, w, h);
+}
+
+
 GtkPrintOperationResult
 _gtk_print_operation_platform_backend_run_dialog (GtkPrintOperation *op,
+						  gboolean           show_dialog,
 						  GtkWindow         *parent,
-						  gboolean          *do_print,
-						  GError           **error)
+						  gboolean          *do_print)
  {
   GtkWidget *pd;
   PrintResponseData rdata;
   gint response;  
-  const char *printer_name;
+  const gchar *printer_name;
    
   rdata.op = op;
   rdata.do_print = FALSE;
   rdata.result = GTK_PRINT_OPERATION_RESULT_CANCEL;
-  rdata.error = error;
   rdata.print_cb = NULL;
   rdata.destroy = NULL;
   rdata.parent = parent;
   rdata.loop = NULL;
 
-  if (op->priv->show_dialog)
+  if (show_dialog)
     {
       pd = get_print_dialog (op, parent);
+
       response = gtk_dialog_run (GTK_DIALOG (pd));
       handle_print_response (pd, response, &rdata);
     }
@@ -439,6 +671,7 @@ _gtk_print_operation_platform_backend_run_dialog (GtkPrintOperation *op,
       GDK_THREADS_ENTER ();  
 
       g_main_loop_unref (rdata.loop);
+      rdata.loop = NULL;
     }
 
   *do_print = rdata.do_print;
@@ -447,7 +680,8 @@ _gtk_print_operation_platform_backend_run_dialog (GtkPrintOperation *op,
 }
 
 
-typedef struct {
+typedef struct 
+{
   GtkPageSetup         *page_setup;
   GtkPageSetupDoneFunc  done_cb;
   gpointer              data;
@@ -459,7 +693,9 @@ page_setup_data_free (gpointer data)
 {
   PageSetupResponseData *rdata = data;
 
-  g_object_unref (rdata->page_setup);
+  if (rdata->page_setup)
+    g_object_unref (rdata->page_setup);
+
   g_free (rdata);
 }
 
@@ -587,11 +823,12 @@ gtk_print_run_page_setup_dialog_async (GtkWindow            *parent,
   gtk_window_present (GTK_WINDOW (dialog));
  }
 
-struct _PrinterFinder {
+struct _PrinterFinder 
+{
   gboolean found_printer;
   GFunc func;
   gpointer data;
-  char *printer_name;
+  gchar *printer_name;
   GList *backends;
   guint timeout_tag;
   GtkPrinter *printer;
@@ -626,8 +863,11 @@ printer_added_cb (GtkPrintBackend *backend,
                   GtkPrinter      *printer, 
 		  PrinterFinder   *finder)
 {
-  if (gtk_printer_is_virtual (printer) ||
-      finder->found_printer)
+  if (finder->found_printer)
+    return;
+
+  /* FIXME this skips "Print to PDF" - is this intentional ? */
+  if (gtk_printer_is_virtual (printer))
     return;
 
   if (finder->printer_name != NULL &&
@@ -668,7 +908,7 @@ printer_list_done_cb (GtkPrintBackend *backend,
 }
 
 static void
-find_printer_init (PrinterFinder *finder,
+find_printer_init (PrinterFinder   *finder,
 		   GtkPrintBackend *backend)
 {
   GList *list;
@@ -696,12 +936,10 @@ find_printer_init (PrinterFinder *finder,
     }
   else
     {
-      g_signal_connect (backend, 
-			"printer-added", 
+      g_signal_connect (backend, "printer-added", 
 			(GCallback) printer_added_cb, 
 			finder);
-      g_signal_connect (backend, 
-			"printer-list-done", 
+      g_signal_connect (backend, "printer-list-done", 
 			(GCallback) printer_list_done_cb, 
 			finder);
     }
@@ -739,9 +977,9 @@ printer_finder_free (PrinterFinder *finder)
 }
 
 static void 
-find_printer (const char *printer,
-	      GFunc func,
-	      gpointer data)
+find_printer (const gchar *printer,
+	      GFunc        func,
+	      gpointer     data)
 {
   GList *node, *next;
   PrinterFinder *finder;
