@@ -35,6 +35,7 @@
 #endif
 
 #include "gtkprogressbar.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkalias.h"
 
@@ -64,8 +65,6 @@ enum {
   PROP_DISCRETE_BLOCKS
 };
 
-static void gtk_progress_bar_class_init    (GtkProgressBarClass *klass);
-static void gtk_progress_bar_init          (GtkProgressBar      *pbar);
 static void gtk_progress_bar_set_property  (GObject             *object,
 					    guint                prop_id,
 					    const GValue        *value,
@@ -74,6 +73,8 @@ static void gtk_progress_bar_get_property  (GObject             *object,
 					    guint                prop_id,
 					    GValue              *value,
 					    GParamSpec          *pspec);
+static gboolean gtk_progress_bar_expose    (GtkWidget           *widget,
+					    GdkEventExpose      *event);
 static void gtk_progress_bar_size_request  (GtkWidget           *widget,
 					    GtkRequisition      *requisition);
 static void gtk_progress_bar_real_update   (GtkProgress         *progress);
@@ -90,33 +91,7 @@ static void gtk_progress_bar_set_activity_blocks_internal (GtkProgressBar *pbar,
 							   guint           blocks);
 
 
-GType
-gtk_progress_bar_get_type (void)
-{
-  static GType progress_bar_type = 0;
-
-  if (!progress_bar_type)
-    {
-      static const GTypeInfo progress_bar_info =
-      {
-	sizeof (GtkProgressBarClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_progress_bar_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkProgressBar),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_progress_bar_init,
-      };
-
-      progress_bar_type =
-	g_type_register_static (GTK_TYPE_PROGRESS, "GtkProgressBar",
-				&progress_bar_info, 0);
-    }
-
-  return progress_bar_type;
-}
+G_DEFINE_TYPE (GtkProgressBar, gtk_progress_bar, GTK_TYPE_PROGRESS)
 
 static void
 gtk_progress_bar_class_init (GtkProgressBarClass *class)
@@ -132,6 +107,7 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   gobject_class->set_property = gtk_progress_bar_set_property;
   gobject_class->get_property = gtk_progress_bar_get_property;
   
+  widget_class->expose_event = gtk_progress_bar_expose;
   widget_class->size_request = gtk_progress_bar_size_request;
 
   progress_class->paint = gtk_progress_bar_paint;
@@ -144,7 +120,7 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
                                                         P_("Adjustment"),
                                                         P_("The GtkAdjustment connected to the progress bar (Deprecated)"),
                                                         GTK_TYPE_ADJUSTMENT,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ORIENTATION,
@@ -153,46 +129,46 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
 						      P_("Orientation and growth direction of the progress bar"),
 						      GTK_TYPE_PROGRESS_BAR_ORIENTATION,
 						      GTK_PROGRESS_LEFT_TO_RIGHT,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_BAR_STYLE,
-                                   g_param_spec_enum ("bar_style",
+                                   g_param_spec_enum ("bar-style",
 						      P_("Bar style"),
 						      P_("Specifies the visual style of the bar in percentage mode (Deprecated)"),
 						      GTK_TYPE_PROGRESS_BAR_STYLE,
 						      GTK_PROGRESS_CONTINUOUS,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ACTIVITY_STEP,
-                                   g_param_spec_uint ("activity_step",
+                                   g_param_spec_uint ("activity-step",
 						      P_("Activity Step"),
 						      P_("The increment used for each iteration in activity mode (Deprecated)"),
-						      -G_MAXUINT,
+						      0,
 						      G_MAXUINT,
 						      3,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ACTIVITY_BLOCKS,
-                                   g_param_spec_uint ("activity_blocks",
+                                   g_param_spec_uint ("activity-blocks",
 						      P_("Activity Blocks"),
 						      P_("The number of blocks which can fit in the progress bar area in activity mode (Deprecated)"),
 						      2,
 						      G_MAXUINT,
 						      5,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_DISCRETE_BLOCKS,
-                                   g_param_spec_uint ("discrete_blocks",
+                                   g_param_spec_uint ("discrete-blocks",
 						      P_("Discrete Blocks"),
 						      P_("The number of discrete blocks in a progress bar (when shown in the discrete style)"),
 						      2,
 						      G_MAXUINT,
 						      10,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
 				   PROP_FRACTION,
@@ -202,17 +178,17 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
 							0.0,
 							1.0,
 							0.0,
-							G_PARAM_READWRITE));  
+							GTK_PARAM_READWRITE));  
   
   g_object_class_install_property (gobject_class,
 				   PROP_PULSE_STEP,
-				   g_param_spec_double ("pulse_step",
+				   g_param_spec_double ("pulse-step",
 							P_("Pulse Step"),
 							P_("The fraction of total progress to move the bouncing block when pulsed"),
 							0.0,
 							1.0,
 							0.1,
-							G_PARAM_READWRITE));  
+							GTK_PARAM_READWRITE));  
   
   g_object_class_install_property (gobject_class,
 				   PROP_TEXT,
@@ -220,7 +196,7 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
 							P_("Text"),
 							P_("Text to be displayed in the progress bar"),
 							"%P %%",
-							G_PARAM_READWRITE));
+							GTK_PARAM_READWRITE));
 
   /**
    * GtkProgressBar:ellipsize:
@@ -243,7 +219,7 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
                                                       P_("The preferred place to ellipsize the string, if the progressbar does not have enough room to display the entire string, if at all"),
 						      PANGO_TYPE_ELLIPSIZE_MODE,
 						      PANGO_ELLIPSIZE_NONE,
-                                                      G_PARAM_READWRITE));
+                                                      GTK_PARAM_READWRITE));
 
 }
 
@@ -465,7 +441,7 @@ gtk_progress_bar_real_update (GtkProgress *progress)
 		}
 	    }
 	}
-      gtk_progress_bar_paint (progress);
+      pbar->dirty = TRUE;
       gtk_widget_queue_draw (GTK_WIDGET (progress));
     }
   else
@@ -478,12 +454,28 @@ gtk_progress_bar_real_update (GtkProgress *progress)
       if (pbar->in_block != in_block)
 	{
 	  pbar->in_block = in_block;
-	  gtk_progress_bar_paint (progress);
+	  pbar->dirty = TRUE;
 	  gtk_widget_queue_draw (GTK_WIDGET (progress));
 	}
     }
 }
 
+static gboolean
+gtk_progress_bar_expose (GtkWidget      *widget,
+		     GdkEventExpose *event)
+{
+  GtkProgressBar *pbar;
+
+  g_return_val_if_fail (GTK_IS_PROGRESS_BAR (widget), FALSE);
+
+  pbar = GTK_PROGRESS_BAR (widget);
+
+  if (GTK_WIDGET_DRAWABLE (widget) && pbar->dirty)
+    gtk_progress_bar_paint (GTK_PROGRESS (pbar));
+
+  return GTK_WIDGET_CLASS (gtk_progress_bar_parent_class)->expose_event (widget, event);
+}
+
 static void
 gtk_progress_bar_size_request (GtkWidget      *widget,
 			       GtkRequisition *requisition)
@@ -542,29 +534,13 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
   if (pbar->orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
       pbar->orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
     {
-      if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
-	{
-	  requisition->width = MAX (MIN_HORIZONTAL_BAR_WIDTH, width);
-	  requisition->height = MAX (MIN_HORIZONTAL_BAR_HEIGHT, height);
-	}
-      else
-	{
-	  requisition->width = MIN_HORIZONTAL_BAR_WIDTH;
-	  requisition->height = MIN_HORIZONTAL_BAR_HEIGHT;
-	}
+      requisition->width = MAX (MIN_HORIZONTAL_BAR_WIDTH, width);
+      requisition->height = MAX (MIN_HORIZONTAL_BAR_HEIGHT, height);
     }
   else
     {
-      if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
-	{	  
-	  requisition->width = MAX (MIN_VERTICAL_BAR_WIDTH, width);
-	  requisition->height = MAX (MIN_VERTICAL_BAR_HEIGHT, height);
-	}
-      else
-	{
-	  requisition->width = MIN_VERTICAL_BAR_WIDTH;
-	  requisition->height = MIN_VERTICAL_BAR_HEIGHT;
-	}
+      requisition->width = MAX (MIN_VERTICAL_BAR_WIDTH, width);
+      requisition->height = MAX (MIN_VERTICAL_BAR_HEIGHT, height);
     }
 }
 
@@ -573,7 +549,6 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
 {
   GtkProgressBar *pbar;
   GtkWidget *widget;
-  gint size;
   GtkProgressBarOrientation orientation;
 
   pbar = GTK_PROGRESS_BAR (progress);
@@ -593,8 +568,6 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
   if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
       orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
     {
-      size = MAX (2, widget->allocation.width / pbar->activity_blocks);
-
       if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT)
 	{
 	  pbar->activity_pos = widget->style->xthickness;
@@ -610,8 +583,6 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
     }
   else
     {
-      size = MAX (2, widget->allocation.height / pbar->activity_blocks);
-
       if (orientation == GTK_PROGRESS_TOP_TO_BOTTOM)
 	{
 	  pbar->activity_pos = widget->style->ythickness;
@@ -633,24 +604,24 @@ gtk_progress_bar_paint_activity (GtkProgressBar            *pbar,
 {
   GtkWidget *widget = GTK_WIDGET (pbar);
   GtkProgress *progress = GTK_PROGRESS (pbar);
-  gint x, y, w, h;
+  GdkRectangle area;
 
   switch (orientation)
     {
     case GTK_PROGRESS_LEFT_TO_RIGHT:
     case GTK_PROGRESS_RIGHT_TO_LEFT:
-      x = pbar->activity_pos;
-      y = widget->style->ythickness;
-      w = MAX (2, widget->allocation.width / pbar->activity_blocks);
-      h = widget->allocation.height - 2 * widget->style->ythickness;
+      area.x = pbar->activity_pos;
+      area.y = widget->style->ythickness;
+      area.width = MAX (2, widget->allocation.width / pbar->activity_blocks);
+      area.height = widget->allocation.height - 2 * widget->style->ythickness;
       break;
 
     case GTK_PROGRESS_TOP_TO_BOTTOM:
     case GTK_PROGRESS_BOTTOM_TO_TOP:
-      x = widget->style->xthickness;
-      y = pbar->activity_pos;
-      w = widget->allocation.width - 2 * widget->style->xthickness;
-      h = MAX (2, widget->allocation.height / pbar->activity_blocks);
+      area.x = widget->style->xthickness;
+      area.y = pbar->activity_pos;
+      area.width = widget->allocation.width - 2 * widget->style->xthickness;
+      area.height = MAX (2, widget->allocation.height / pbar->activity_blocks);
       break;
 
     default:
@@ -661,8 +632,8 @@ gtk_progress_bar_paint_activity (GtkProgressBar            *pbar,
   gtk_paint_box (widget->style,
 		 progress->offscreen_pixmap,
 		 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-		 NULL, widget, "bar",
-		 x, y, w, h);
+		 &area, widget, "bar",
+		 area.x, area.y, area.width, area.height);
 }
 
 static void
@@ -670,8 +641,8 @@ gtk_progress_bar_paint_continuous (GtkProgressBar            *pbar,
 				   gint                       amount,
 				   GtkProgressBarOrientation  orientation)
 {
+  GdkRectangle area;
   GtkWidget *widget = GTK_WIDGET (pbar);
-  gint x, y, w, h;
 
   if (amount <= 0)
     return;
@@ -680,24 +651,24 @@ gtk_progress_bar_paint_continuous (GtkProgressBar            *pbar,
     {
     case GTK_PROGRESS_LEFT_TO_RIGHT:
     case GTK_PROGRESS_RIGHT_TO_LEFT:
-      w = amount;
-      h = widget->allocation.height - widget->style->ythickness * 2;
-      y = widget->style->ythickness;
+      area.width = amount;
+      area.height = widget->allocation.height - widget->style->ythickness * 2;
+      area.y = widget->style->ythickness;
       
-      x = widget->style->xthickness;
+      area.x = widget->style->xthickness;
       if (orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
-	x = widget->allocation.width - amount - x;
+	area.x = widget->allocation.width - amount - area.x;
       break;
       
     case GTK_PROGRESS_TOP_TO_BOTTOM:
     case GTK_PROGRESS_BOTTOM_TO_TOP:
-      w = widget->allocation.width - widget->style->xthickness * 2;
-      h = amount;
-      x = widget->style->xthickness;
+      area.width = widget->allocation.width - widget->style->xthickness * 2;
+      area.height = amount;
+      area.x = widget->style->xthickness;
       
-      y = widget->style->ythickness;
+      area.y = widget->style->ythickness;
       if (orientation == GTK_PROGRESS_BOTTOM_TO_TOP)
-	y = widget->allocation.height - amount - y;
+	area.y = widget->allocation.height - amount - area.y;
       break;
       
     default:
@@ -708,8 +679,8 @@ gtk_progress_bar_paint_continuous (GtkProgressBar            *pbar,
   gtk_paint_box (widget->style,
 		 GTK_PROGRESS (pbar)->offscreen_pixmap,
 		 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-		 NULL, widget, "bar",
-		 x, y, w, h);
+		 &area, widget, "bar",
+		 area.x, area.y, area.width, area.height);
 }
 
 static void
@@ -721,7 +692,8 @@ gtk_progress_bar_paint_discrete (GtkProgressBar            *pbar,
 
   for (i = 0; i <= pbar->in_block; i++)
     {
-      gint x, y, w, h, space;
+      GdkRectangle area;
+      gint space;
 
       switch (orientation)
 	{
@@ -729,26 +701,26 @@ gtk_progress_bar_paint_discrete (GtkProgressBar            *pbar,
 	case GTK_PROGRESS_RIGHT_TO_LEFT:
 	  space = widget->allocation.width - 2 * widget->style->xthickness;
 	  
-	  x = widget->style->xthickness + (i * space) / pbar->blocks;
-	  y = widget->style->ythickness;
-	  w = widget->style->xthickness + ((i + 1) * space) / pbar->blocks - x;
-	  h = widget->allocation.height - 2 * widget->style->ythickness;
+	  area.x = widget->style->xthickness + (i * space) / pbar->blocks;
+	  area.y = widget->style->ythickness;
+	  area.width = widget->style->xthickness + ((i + 1) * space) / pbar->blocks - area.x;
+	  area.height = widget->allocation.height - 2 * widget->style->ythickness;
 
 	  if (orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
-	    x = widget->allocation.width - w - x;
+	    area.x = widget->allocation.width - area.width - area.x;
 	  break;
 	  
 	case GTK_PROGRESS_TOP_TO_BOTTOM:
 	case GTK_PROGRESS_BOTTOM_TO_TOP:
 	  space = widget->allocation.height - 2 * widget->style->ythickness;
 	  
-	  x = widget->style->xthickness;
-	  y = widget->style->ythickness + (i * space) / pbar->blocks;
-	  w = widget->allocation.width - 2 * widget->style->xthickness;
-	  h = widget->style->ythickness + ((i + 1) * space) / pbar->blocks - y;
+	  area.x = widget->style->xthickness;
+	  area.y = widget->style->ythickness + (i * space) / pbar->blocks;
+	  area.width = widget->allocation.width - 2 * widget->style->xthickness;
+	  area.height = widget->style->ythickness + ((i + 1) * space) / pbar->blocks - area.y;
 	  
 	  if (orientation == GTK_PROGRESS_BOTTOM_TO_TOP)
-	    y = widget->allocation.height - h - y;
+	    area.y = widget->allocation.height - area.height - area.y;
 	  break;
 
 	default:
@@ -759,8 +731,8 @@ gtk_progress_bar_paint_discrete (GtkProgressBar            *pbar,
       gtk_paint_box (widget->style,
 		     GTK_PROGRESS (pbar)->offscreen_pixmap,
 		     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-		     NULL, widget, "bar",
-		     x, y, w, h);
+		     &area, widget, "bar",
+		     area.x, area.y, area.width, area.height);
     }
 }
 
@@ -917,6 +889,8 @@ gtk_progress_bar_paint (GtkProgress *progress)
 	  else
 	    gtk_progress_bar_paint_discrete (pbar, orientation);
 	}
+
+      pbar->dirty = FALSE;
     }
 }
 
@@ -1037,7 +1011,7 @@ gtk_progress_bar_pulse (GtkProgressBar *pbar)
 /**
  * gtk_progress_bar_set_text:
  * @pbar: a #GtkProgressBar
- * @text: a UTF-8 string
+ * @text: a UTF-8 string, or %NULL 
  * 
  * Causes the given @text to appear superimposed on the progress bar.
  **/
