@@ -152,16 +152,30 @@ struct SortInfo {
 static gint
 node_compare (gconstpointer n1, gconstpointer n2, gpointer data)
 {
-  SortInfo *info = data;
+  const SortInfo *info = data;
   const GtkSequenceNode *node1 = n1;
   const GtkSequenceNode *node2 = n2;
+  gint retval;
   
   if (node1->is_end)
+      return 1;
+
+  if (node2->is_end)
+      return -1;
+
+  retval = (* info->cmp) (node1, node2, info->data);
+
+  /* If the nodes are different, but the user-supplied compare function
+   * compares them equal, then force an arbitrary (but consistent) order
+   * on them, so that our sorts will be stable
+   */
+  if (retval != 0 || n1 == n2)
+    return retval;
+  
+  if (n1 > n2)
     return 1;
-  else if (node2->is_end)
-    return -1;
   else
-    return (* info->cmp) (node1, node2, info->data);
+    return -1;
 }
 
 void
@@ -454,9 +468,10 @@ _gtk_sequence_sort_changed  (GtkSequencePtr	     ptr,
   
 {
   GtkSequence *seq;
-  
+
+  g_return_if_fail (ptr != NULL);
   g_return_if_fail (!ptr->is_end);
-  
+
   seq = _gtk_sequence_node_get_sequence (ptr);
   _gtk_sequence_unlink (seq, ptr);
   _gtk_sequence_node_insert_sorted (seq->node, ptr, cmp_func, cmp_data);
@@ -1008,11 +1023,7 @@ _gtk_sequence_node_insert_sorted (GtkSequenceNode *node,
   
   if (node_compare (new, closest, &info) > 0)
     closest = _gtk_sequence_node_next (closest);
-  
-  /* this can never fail since we have a bigger-than-everything
-   * end-node
-   */
-  g_assert (node_compare (new, closest, &info) <= 0);
+
   _gtk_sequence_node_insert_before (closest, new);
 }
 
@@ -1070,21 +1081,35 @@ void
 _gtk_sequence_swap (GtkSequencePtr a,
 		    GtkSequencePtr b)
 {
-  GtkSequenceNode temp;
-  gpointer temp_data;
+  GtkSequenceNode *leftmost, *rightmost, *rightmost_next;
+  int a_pos, b_pos;
   
   g_return_if_fail (!_gtk_sequence_ptr_is_end (a));
   g_return_if_fail (!_gtk_sequence_ptr_is_end (b));
+
+  if (a == b)
+      return;
+
+  a_pos = _gtk_sequence_ptr_get_position (a);
+  b_pos = _gtk_sequence_ptr_get_position (b);
+
+  if (a_pos > b_pos)
+    {
+      leftmost = b;
+      rightmost = a;
+    }
+  else
+    {
+      leftmost = a;
+      rightmost = b;
+    }
+
+  rightmost_next = _gtk_sequence_node_next (rightmost);
+
+  /* Situation now:  ..., leftmost, ......., rightmost, rightmost_next, ... */
   
-  /* swap contents of the nodes */
-  temp = *a;
-  *a = *b;
-  *b = temp;
-  
-  /* swap data back */
-  temp_data = a->data;
-  a->data = b->data;
-  b->data = temp_data;
+  _gtk_sequence_move (rightmost, leftmost);
+  _gtk_sequence_move (leftmost, rightmost_next);
 }
 
 void
