@@ -1167,6 +1167,22 @@ gtk_unit_get_type (void)
   }
   return etype;
 }
+GType
+gtk_tree_view_grid_lines_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GTK_TREE_VIEW_GRID_LINES_NONE, "GTK_TREE_VIEW_GRID_LINES_NONE", "none" },
+      { GTK_TREE_VIEW_GRID_LINES_HORIZONTAL, "GTK_TREE_VIEW_GRID_LINES_HORIZONTAL", "horizontal" },
+      { GTK_TREE_VIEW_GRID_LINES_VERTICAL, "GTK_TREE_VIEW_GRID_LINES_VERTICAL", "vertical" },
+      { GTK_TREE_VIEW_GRID_LINES_BOTH, "GTK_TREE_VIEW_GRID_LINES_BOTH", "both" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static (g_intern_static_string ("GtkTreeViewGridLines"), values);
+  }
+  return etype;
+}
 
 /* enumerations from "gtkfilechooser.h" */
 GType
@@ -1426,7 +1442,7 @@ gtk_print_operation_result_get_type (void)
       { GTK_PRINT_OPERATION_RESULT_ERROR, "GTK_PRINT_OPERATION_RESULT_ERROR", "error" },
       { GTK_PRINT_OPERATION_RESULT_APPLY, "GTK_PRINT_OPERATION_RESULT_APPLY", "apply" },
       { GTK_PRINT_OPERATION_RESULT_CANCEL, "GTK_PRINT_OPERATION_RESULT_CANCEL", "cancel" },
-      { GTK_PRINT_OPERATION_RESULT_PREVIEW, "GTK_PRINT_OPERATION_RESULT_PREVIEW", "preview" },
+      { GTK_PRINT_OPERATION_RESULT_IN_PROGRESS, "GTK_PRINT_OPERATION_RESULT_IN_PROGRESS", "in-progress" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static (g_intern_static_string ("GtkPrintOperationResult"), values);
@@ -1434,6 +1450,22 @@ gtk_print_operation_result_get_type (void)
   return etype;
 }
 GType
+gtk_print_operation_action_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, "GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG", "print-dialog" },
+      { GTK_PRINT_OPERATION_ACTION_PRINT, "GTK_PRINT_OPERATION_ACTION_PRINT", "print" },
+      { GTK_PRINT_OPERATION_ACTION_PREVIEW, "GTK_PRINT_OPERATION_ACTION_PREVIEW", "preview" },
+      { GTK_PRINT_OPERATION_ACTION_EXPORT, "GTK_PRINT_OPERATION_ACTION_EXPORT", "export" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static (g_intern_static_string ("GtkPrintOperationAction"), values);
+  }
+  return etype;
+}
+GType
 gtk_print_error_get_type (void)
 {
   static GType etype = 0;
