@@ -1,3 +1,4 @@
+#include <math.h>
 #include <pango/pangocairo.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkprintoperation.h>
@@ -245,6 +246,8 @@ typedef struct {
   char *text;
   PangoLayout *layout;
   GList *page_breaks;
+  GtkWidget *font_button;
+  char *font;
 } PrintData;
 
 static void
@@ -263,9 +266,9 @@ begin_print (GtkPrintOperation *operation,
   width = gtk_print_context_get_width (context);
   height = gtk_print_context_get_height (context);
 
-  print_data->layout = gtk_print_context_create_layout (context);
+  print_data->layout = gtk_print_context_create_pango_layout (context);
 
-  desc = pango_font_description_from_string ("Sans 12");
+  desc = pango_font_description_from_string (print_data->font);
   pango_layout_set_font_description (print_data->layout, desc);
   pango_font_description_free (desc);
 
@@ -301,7 +304,6 @@ begin_print (GtkPrintOperation *operation,
   gtk_print_operation_set_n_pages (operation, g_list_length (page_breaks) + 1);
   
   print_data->page_breaks = page_breaks;
-  
 }
 
 static void
@@ -315,6 +317,7 @@ draw_page (GtkPrintOperation *operation,
   int start, end, i;
   PangoLayoutIter *iter;
   double start_pos;
+
   if (page_nr == 0)
     start = 0;
   else
@@ -329,7 +332,7 @@ draw_page (GtkPrintOperation *operation,
   else
     end = GPOINTER_TO_INT (pagebreak->data);
     
-  cr = gtk_print_context_get_cairo (context);
+  cr = gtk_print_context_get_cairo_context (context);
 
   cairo_set_source_rgb (cr, 0, 0, 0);
   
@@ -391,16 +394,271 @@ status_changed_cb (GtkPrintOperation *op,
   update_statusbar ();
 }
 
+static GtkWidget *
+create_custom_widget (GtkPrintOperation *operation,
+		      PrintData *data)
+{
+  GtkWidget *vbox, *hbox, *font, *label;
+
+  gtk_print_operation_set_custom_tab_label (operation, "Other");
+  vbox = gtk_vbox_new (FALSE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
+
+  hbox = gtk_hbox_new (FALSE, 8);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+  gtk_widget_show (hbox);
+
+  label = gtk_label_new ("Font:");
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_widget_show (label);
+  
+  font = gtk_font_button_new_with_font  (data->font);
+  gtk_box_pack_start (GTK_BOX (hbox), font, FALSE, FALSE, 0);
+  gtk_widget_show (font);
+  data->font_button = font;
+
+  return vbox;
+}
+
+static void
+custom_widget_apply (GtkPrintOperation *operation,
+		     GtkWidget *widget,
+		     PrintData *data)
+{
+  const char *selected_font;
+  selected_font = gtk_font_button_get_font_name  (GTK_FONT_BUTTON (data->font_button));
+  g_free (data->font);
+  data->font = g_strdup (selected_font);
+}
+
+typedef struct 
+{
+  GtkPrintOperation *op;
+  GtkPrintOperationPreview *preview;
+  GtkWidget         *spin;
+  GtkWidget         *area;
+  gint               page;
+  PrintData *data;
+  gdouble dpi_x, dpi_y;
+} PreviewOp;
+
+static gboolean
+preview_expose (GtkWidget      *widget,
+		GdkEventExpose *event,
+		gpointer        data)
+{
+  PreviewOp *pop = data;
+
+  gdk_window_clear (pop->area->window);
+  gtk_print_operation_preview_render_page (pop->preview,
+					   pop->page - 1);
+
+  return TRUE;
+}
+
+static void
+preview_ready (GtkPrintOperationPreview *preview,
+	       GtkPrintContext          *context,
+	       gpointer                  data)
+{
+  PreviewOp *pop = data;
+  gint n_pages;
+
+  g_object_get (pop->op, "n-pages", &n_pages, NULL);
+
+  gtk_spin_button_set_range (GTK_SPIN_BUTTON (pop->spin), 
+			     1.0, n_pages);
+
+  g_signal_connect (pop->area, "expose_event",
+		    G_CALLBACK (preview_expose),
+		    pop);
+
+  gtk_widget_queue_draw (pop->area);
+}
+
+static void
+preview_got_page_size (GtkPrintOperationPreview *preview, 
+		       GtkPrintContext          *context,
+		       GtkPageSetup             *page_setup,
+		       gpointer                  data)
+{
+  PreviewOp *pop = data;
+  GtkPaperSize *paper_size;
+  double w, h;
+  cairo_t *cr;
+  gdouble dpi_x, dpi_y;
+
+  paper_size = gtk_page_setup_get_paper_size (page_setup);
+
+  w = gtk_paper_size_get_width (paper_size, GTK_UNIT_INCH);
+  h = gtk_paper_size_get_height (paper_size, GTK_UNIT_INCH);
+
+  cr = gdk_cairo_create (pop->area->window);
+
+  dpi_x = pop->area->allocation.width/w;
+  dpi_y = pop->area->allocation.height/h;
+
+  if (fabs (dpi_x - pop->dpi_x) > 0.001 ||
+      fabs (dpi_y - pop->dpi_y) > 0.001)
+    {
+      gtk_print_context_set_cairo_context (context, cr, dpi_x, dpi_y);
+      pop->dpi_x = dpi_x;
+      pop->dpi_y = dpi_y;
+    }
+
+  pango_cairo_update_layout (cr, pop->data->layout);
+  cairo_destroy (cr);
+}
+
+static void
+update_page (GtkSpinButton *widget,
+	     gpointer       data)
+{
+  PreviewOp *pop = data;
+
+  pop->page = gtk_spin_button_get_value_as_int (widget);
+  gtk_widget_queue_draw (pop->area);
+}
+
+static void
+preview_destroy (GtkWindow *window, 
+		 PreviewOp *pop)
+{
+  gtk_print_operation_preview_end_preview (pop->preview);
+  g_object_unref (pop->op);
+
+  g_free (pop);
+}
+
+static gboolean 
+do_preview (GtkPrintOperation        *op,
+	    GtkPrintOperationPreview *preview,
+	    GtkPrintContext          *context,
+	    GtkWindow                *parent,
+	    gpointer                  data)
+{
+  GtkPrintSettings *settings;
+  GtkWidget *window, *close, *page, *hbox, *vbox, *da;
+  gdouble width, height;
+  cairo_t *cr;
+  PreviewOp *pop;
+  PrintData *print_data = data;
+
+  pop = g_new0 (PreviewOp, 1);
+
+  pop->data = print_data;
+  settings = gtk_print_operation_get_print_settings (op);
+
+  width = 200;
+  height = 300;
+  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_transient_for (GTK_WINDOW (window), 
+				GTK_WINDOW (main_window));
+  vbox = gtk_vbox_new (FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (window), vbox);
+  hbox = gtk_hbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox,
+		      FALSE, FALSE, 0);
+  page = gtk_spin_button_new_with_range (1, 100, 1);
+  gtk_box_pack_start (GTK_BOX (hbox), page, FALSE, FALSE, 0);
+  
+  close = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
+  gtk_box_pack_start (GTK_BOX (hbox), close, FALSE, FALSE, 0);
+
+  da = gtk_drawing_area_new ();
+  gtk_widget_set_size_request (GTK_WIDGET (da), width, height);
+  gtk_box_pack_start (GTK_BOX (vbox), da, TRUE, TRUE, 0);
+
+  gtk_widget_set_double_buffered (da, FALSE);
+
+  gtk_widget_realize (da);
+  
+  cr = gdk_cairo_create (da->window);
+
+  /* TODO: What dpi to use here? This will be used for pagination.. */
+  gtk_print_context_set_cairo_context (context, cr, 72, 72);
+  cairo_destroy (cr);
+  
+  pop->op = op;
+  pop->preview = preview;
+  pop->spin = page;
+  pop->area = da;
+  pop->page = 1;
+
+  g_signal_connect (page, "value-changed", 
+		    G_CALLBACK (update_page), pop);
+  g_signal_connect_swapped (close, "clicked", 
+			    G_CALLBACK (gtk_widget_destroy), window);
+
+  g_signal_connect (preview, "ready",
+		    G_CALLBACK (preview_ready), pop);
+  g_signal_connect (preview, "got-page-size",
+		    G_CALLBACK (preview_got_page_size), pop);
+
+  g_signal_connect (window, "destroy", 
+                    G_CALLBACK (preview_destroy), pop);
+                            
+  gtk_widget_show_all (window);
+  
+  return TRUE;
+}
+
+static void
+print_done (GtkPrintOperation *op,
+	    GtkPrintOperationResult res,
+	    PrintData *print_data)
+{
+  GError *error = NULL;
+
+  if (res == GTK_PRINT_OPERATION_RESULT_ERROR)
+    {
+
+      GtkWidget *error_dialog;
+      
+      gtk_print_operation_get_error (op, &error);
+      
+      error_dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
+					     GTK_DIALOG_DESTROY_WITH_PARENT,
+					     GTK_MESSAGE_ERROR,
+					     GTK_BUTTONS_CLOSE,
+					     "Error printing file:\n%s",
+					     error ? error->message : "no details");
+      g_signal_connect (error_dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+      gtk_widget_show (error_dialog);
+    }
+  else if (res == GTK_PRINT_OPERATION_RESULT_APPLY)
+    {
+      if (settings != NULL)
+	g_object_unref (settings);
+      settings = g_object_ref (gtk_print_operation_get_print_settings (op));
+    }
+
+  g_free (print_data->text);
+  g_free (print_data->font);
+  g_free (print_data);
+  
+  if (!gtk_print_operation_is_finished (op))
+    {
+      g_object_ref (op);
+      active_prints = g_list_append (active_prints, op);
+      update_statusbar ();
+      
+      /* This ref is unref:ed when we get the final state change */
+      g_signal_connect (op, "status_changed",
+			G_CALLBACK (status_changed_cb), NULL);
+    }
+}
+
 static void
 do_print (GtkAction *action)
 {
-  GtkWidget *error_dialog;
   GtkPrintOperation *print;
-  PrintData print_data;
-  GtkPrintOperationResult res;
-  GError *error;
+  PrintData *print_data;
 
-  print_data.text = get_text ();
+  print_data = g_new0 (PrintData, 1);
+
+  print_data->text = get_text ();
+  print_data->font = g_strdup ("Sans 12");
 
   print = gtk_print_operation_new ();
 
@@ -412,42 +670,21 @@ do_print (GtkAction *action)
   if (page_setup != NULL)
     gtk_print_operation_set_default_page_setup (print, page_setup);
   
-  g_signal_connect (print, "begin_print", G_CALLBACK (begin_print), &print_data);
-  g_signal_connect (print, "draw_page", G_CALLBACK (draw_page), &print_data);
+  g_signal_connect (print, "begin_print", G_CALLBACK (begin_print), print_data);
+  g_signal_connect (print, "draw_page", G_CALLBACK (draw_page), print_data);
+  g_signal_connect (print, "create_custom_widget", G_CALLBACK (create_custom_widget), print_data);
+  g_signal_connect (print, "custom_widget_apply", G_CALLBACK (custom_widget_apply), print_data);
+  g_signal_connect (print, "preview", G_CALLBACK (do_preview), print_data);
 
-  error = NULL;
-  res = gtk_print_operation_run (print, GTK_WINDOW (main_window), &error);
+  g_signal_connect (print, "done", G_CALLBACK (print_done), print_data);
 
-  if (res == GTK_PRINT_OPERATION_RESULT_ERROR)
-    {
-      error_dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
-					     GTK_DIALOG_DESTROY_WITH_PARENT,
-					     GTK_MESSAGE_ERROR,
-					     GTK_BUTTONS_CLOSE,
-					     "Error printing file:\n%s",
-					     error->message);
-      g_signal_connect (error_dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
-      gtk_widget_show (error_dialog);
-      g_error_free (error);
-    }
-  else if (res == GTK_PRINT_OPERATION_RESULT_APPLY)
-    {
-      if (settings != NULL)
-	g_object_unref (settings);
-      settings = g_object_ref (gtk_print_operation_get_print_settings (print));
-    }
+  gtk_print_operation_set_export_filename (print, "test.pdf");
+
+#if 0
+  gtk_print_operation_set_allow_async (print, TRUE);
+#endif
+  gtk_print_operation_run (print, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, GTK_WINDOW (main_window), NULL);
 
-  if (!gtk_print_operation_is_finished (print))
-    {
-      g_object_ref (print);
-      active_prints = g_list_append (active_prints, print);
-      update_statusbar ();
-      
-      /* This ref is unref:ed when we get the final state change */
-      g_signal_connect (print, "status_changed",
-			G_CALLBACK (status_changed_cb), NULL);
-    }
-  
   g_object_unref (print);
 }
 
@@ -671,6 +908,7 @@ create_window (void)
 int
 main (int argc, char **argv)
 {
+  g_set_application_name ("Print editor");
   gtk_init (&argc, &argv);
 
   create_window ();
